@@ -10,11 +10,13 @@ from aniso8601.builders import TupleBuilder
 from aniso8601.builders.python import PythonTimeBuilder
 from aniso8601.compat import is_string
 from aniso8601.date import parse_date
-from aniso8601.decimalfraction import find_separator, normalize
+from aniso8601.decimalfraction import normalize
 from aniso8601.exceptions import ISOFormatError
 from aniso8601.resolution import TimeResolution
 from aniso8601.timezone import parse_timezone
 
+TIMEZONE_DELIMITERS = ['Z', '+', '-']
+
 def get_time_resolution(isotimestr):
     #Valid time formats are:
     #
@@ -43,34 +45,26 @@ def get_time_resolution(isotimestr):
     #hh:mm±hh
     #hhmm±hh
     #hh±hh
-    if is_string(isotimestr) is False:
-        raise ValueError('Time must be string.')
+    isotimetuple = parse_time(isotimestr, builder=TupleBuilder)
 
-    timestr = _split_tz(isotimestr)[0].replace(':', '')
+    return _get_time_resolution(isotimetuple)
 
-    if (len(timestr) == 0 or
-            timestr[0].isdigit() is False or
-            timestr[-1].isdigit() is False):
-        raise ISOFormatError('"{0}" is not a valid ISO 8601 time.'
-                             .format(timestr))
+def get_datetime_resolution(isodatetimestr, delimiter='T'):
+    #<date>T<time>
+    #
+    #Time part cannot be omittted so return time resolution
+    isotimetuple = parse_datetime(isodatetimestr, delimiter=delimiter, builder=TupleBuilder).time
 
-    #Format must be hhmmss, hhmm, or hh
-    timestrlen = find_separator(timestr)
-    if timestrlen == -1:
-        timestrlen = len(timestr)
+    return _get_time_resolution(isotimetuple)
 
-    if timestrlen == 6:
-        #hhmmss
+def _get_time_resolution(isotimetuple):
+    if isotimetuple.ss is not None:
         return TimeResolution.Seconds
-    elif timestrlen == 4:
-        #hhmm
-        return TimeResolution.Minutes
-    elif timestrlen == 2:
-        #hh
-        return TimeResolution.Hours
 
-    raise ISOFormatError('"{0}" is not a valid ISO 8601 time.'
-                         .format(isotimestr))
+    if isotimetuple.mm is not None:
+        return TimeResolution.Minutes
+
+    return TimeResolution.Hours
 
 def parse_time(isotimestr, builder=PythonTimeBuilder):
     #Given a string in any ISO 8601 time format, return a datetime.time object
@@ -102,16 +96,82 @@ def parse_time(isotimestr, builder=PythonTimeBuilder):
     #hh:mm±hh
     #hhmm±hh
     #hh±hh
-    timeresolution = get_time_resolution(isotimestr)
+    if is_string(isotimestr) is False:
+        raise ValueError('Time must be string.')
 
-    (timestr, tzstr) = _split_tz(isotimestr)
+    if len(isotimestr) == 0:
+        raise ISOFormatError('"{0}" is not a valid ISO 8601 time.'
+                             .format(isotimestr))
+
+    timestr = normalize(isotimestr)
+
+    hourstr = None
+    minutestr = None
+    secondstr = None
+    tzstr = None
+
+    fractionalstr = None
+
+    #Split out the timezone
+    for delimiter in TIMEZONE_DELIMITERS:
+        delimiteridx = timestr.find(delimiter)
+
+        if delimiteridx != -1:
+            tzstr = timestr[delimiteridx:]
+            timestr = timestr[0:delimiteridx]
+
+    #Split out the fractional component
+    if timestr.find('.') != -1:
+        timestr, fractionalstr = timestr.split('.', 1)
+
+        if fractionalstr.isdigit() is False:
+            raise ISOFormatError('"{0}" is not a valid ISO 8601 time.'
+                                 .format(isotimestr))
+
+
+    if len(timestr) == 2:
+        #hh
+        hourstr = timestr
+    elif len(timestr) == 4 or len(timestr) == 5:
+        #hh:mm
+        #hhmm
+        if timestr.count(':') == 1:
+            hourstr, minutestr = timestr.split(':')
+        else:
+            hourstr = timestr[0:2]
+            minutestr = timestr[2:]
+    elif len(timestr) == 6 or len(timestr) == 8:
+        #hh:mm:ss
+        #hhmmss
+        if timestr.count(':') == 2:
+            hourstr, minutestr, secondstr = timestr.split(':')
+        else:
+            hourstr = timestr[0:2]
+            minutestr = timestr[2:4]
+            secondstr = timestr[4:]
+    else:
+        raise ISOFormatError('"{0}" is not a valid ISO 8601 time.'
+                             .format(isotimestr))
+
+    for componentstr in [hourstr, minutestr, secondstr]:
+        if componentstr is not None and componentstr.isdigit() is False:
+            raise ISOFormatError('"{0}" is not a valid ISO 8601 time.'
+                                 .format(isotimestr))
+
+    if fractionalstr is not None:
+        if secondstr is not None:
+            secondstr = secondstr + '.' + fractionalstr
+        elif minutestr is not None:
+            minutestr = minutestr + '.' + fractionalstr
+        else:
+            hourstr = hourstr + '.' + fractionalstr
 
     if tzstr is None:
         tz = None
     else:
         tz = parse_timezone(tzstr, builder=TupleBuilder)
 
-    return _RESOLUTION_MAP[timeresolution](timestr, tz, builder)
+    return builder.build_time(hh=hourstr, mm=minutestr, ss=secondstr, tz=tz)
 
 def parse_datetime(isodatetimestr, delimiter='T', builder=PythonTimeBuilder):
     #Given a string in ISO 8601 date time format, return a datetime.datetime
@@ -124,7 +184,7 @@ def parse_datetime(isodatetimestr, delimiter='T', builder=PythonTimeBuilder):
 
     if delimiter not in isodatetimestr:
         raise ISOFormatError('Delimiter "{0}" is not in combined date time '
-                              'string "{1}".'
+                             'string "{1}".'
                              .format(delimiter, isodatetimestr))
 
     isodatestr, isotimestr = isodatetimestr.split(delimiter, 1)
@@ -134,60 +194,3 @@ def parse_datetime(isodatetimestr, delimiter='T', builder=PythonTimeBuilder):
     timepart = parse_time(isotimestr, builder=TupleBuilder)
 
     return builder.build_datetime(datepart, timepart)
-
-def _parse_hour(timestr, tz, builder):
-    #Format must be hh or hh.
-    hourstr = timestr
-
-    if hourstr == '24':
-        return builder.build_time(tz=tz)
-
-    return builder.build_time(hh=normalize(hourstr), tz=tz)
-
-def _parse_minute_time(timestr, tz, builder):
-    #Format must be hhmm, hhmm., hh:mm or hh:mm.
-    if timestr.count(':') == 1:
-        #hh:mm or hh:mm.
-        hourstr, minutestr = timestr.split(':')
-    else:
-        #hhmm or hhmm.
-        hourstr = timestr[0:2]
-        minutestr = timestr[2:]
-
-    return builder.build_time(hh=normalize(hourstr), mm=normalize(minutestr), tz=tz)
-
-def _parse_second_time(timestr, tz, builder):
-    #Format must be hhmmss, hhmmss., hh:mm:ss or hh:mm:ss.
-    if timestr.count(':') == 2:
-        #hh:mm:ss or hh:mm:ss.
-        hourstr, minutestr, secondstr = timestr.split(':')
-    else:
-        #hhmmss or hhmmss.
-        hourstr = timestr[0:2]
-        minutestr = timestr[2:4]
-        secondstr = timestr[4:]
-
-    return builder.build_time(hh=normalize(hourstr), mm=normalize(minutestr),
-                              ss=normalize(secondstr), tz=tz)
-
-def _split_tz(isotimestr):
-    if isotimestr.find('+') != -1:
-        timestr = isotimestr[0:isotimestr.find('+')]
-        tzstr = isotimestr[isotimestr.find('+'):]
-    elif isotimestr.find('-') != -1:
-        timestr = isotimestr[0:isotimestr.find('-')]
-        tzstr = isotimestr[isotimestr.find('-'):]
-    elif isotimestr.endswith('Z'):
-        timestr = isotimestr[:-1]
-        tzstr = 'Z'
-    else:
-        timestr = isotimestr
-        tzstr = None
-
-    return (timestr, tzstr)
-
-_RESOLUTION_MAP = {
-    TimeResolution.Hours: _parse_hour,
-    TimeResolution.Minutes: _parse_minute_time,
-    TimeResolution.Seconds: _parse_second_time
-}
