@@ -1,21 +1,21 @@
 # -*- coding: utf-8 -*-
 
-# Copyright (c) 2018, Brandon Nielsen
+# Copyright (c) 2021, Brandon Nielsen
 # All rights reserved.
 #
 # This software may be modified and distributed under the terms
 # of the BSD license.  See the LICENSE file for details.
 
-from aniso8601.builder import PythonTimeBuilder
+from aniso8601.builders.python import PythonTimeBuilder
+from aniso8601.compat import is_string
 from aniso8601.exceptions import ISOFormatError
 
 def parse_timezone(tzstr, builder=PythonTimeBuilder):
     #tzstr can be Z, ±hh:mm, ±hhmm, ±hh
-    if 'Z' in tzstr:
-        if len(tzstr) != 1:
-            raise ISOFormatError('"{0}" is not a valid ISO 8601 time offset.'
-                                 .format(tzstr))
+    if is_string(tzstr) is False:
+        raise ValueError('Time zone must be string.')
 
+    if len(tzstr) == 1 and tzstr[0] == 'Z':
         return builder.build_timezone(negative=False, Z=True, name=tzstr)
     elif len(tzstr) == 6:
         #±hh:mm
@@ -45,10 +45,17 @@ def parse_timezone(tzstr, builder=PythonTimeBuilder):
         raise ISOFormatError('"{0}" is not a valid ISO 8601 time offset.'
                              .format(tzstr))
 
+    for componentstr in [hourstr, minutestr]:
+        if componentstr is not None:
+            if componentstr.isdigit() is False:
+                raise ISOFormatError('"{0}" is not a valid ISO 8601 time offset.'
+                                     .format(tzstr))
+
     if tzstr[0] == '+':
         return builder.build_timezone(negative=False, hh=hourstr,
                                       mm=minutestr, name=tzstr)
-    elif tzstr[0] == '-':
+
+    if tzstr[0] == '-':
         return builder.build_timezone(negative=True, hh=hourstr,
                                       mm=minutestr, name=tzstr)
 
